@@ -49,7 +49,7 @@ def find_scoreboard_player_type(
         return ScoreboardPlayer(
             player_type=PlayerType.INTEGER, value=int(token.string))
 
-    splits = token.string.split(':')
+    splits = token.string.split(":")
     if len(splits) == 1:
         if allow_integer:
             raise JMCSyntaxException(
@@ -88,7 +88,7 @@ class ArgType(Enum):
 
 
 class Arg:
-    __slots__ = ('token', 'arg_type')
+    __slots__ = ("token", "arg_type")
 
     def __init__(self, token: Token, arg_type: ArgType) -> None:
         self.token = token
@@ -109,6 +109,18 @@ class Arg:
         if verifier == ArgType.SCOREBOARD_PLAYER:
             if self.arg_type in {ArgType.SCOREBOARD, ArgType.INTEGER}:
                 return self
+            if self.arg_type == ArgType.KEYWORD and self.token.string.count(
+                    ":") == 1:
+                return self
+            raise JMCValueError(
+                f"For '{key_string}' key, expected {verifier.value}, got {self.arg_type.value}", self.token, tokenizer)
+        if verifier == ArgType.SCOREBOARD:
+            if self.arg_type == ArgType.SCOREBOARD:
+                return self
+            if self.arg_type == ArgType.KEYWORD and self.token.string.count(
+                    ":") == 1:
+                self.arg_type = ArgType.SCOREBOARD
+                return self
             raise JMCValueError(
                 f"For '{key_string}' key, expected {verifier.value}, got {self.arg_type.value}", self.token, tokenizer)
         if verifier == ArgType.FLOAT:
@@ -124,13 +136,21 @@ class Arg:
                 return self
             raise JMCValueError(
                 f"For '{key_string}' key, expected {verifier.value}, got {self.arg_type.value}", self.token, tokenizer)
+        if verifier == ArgType.SELECTOR:
+            if self.arg_type == ArgType.SELECTOR:
+                return self
+            if self.arg_type == ArgType.KEYWORD:
+                self.arg_type = ArgType.SELECTOR
+                return self
+            raise JMCValueError(
+                f"For '{key_string}' key, expected {verifier.value}, got {self.arg_type.value}", self.token, tokenizer)
         if verifier == ArgType.KEYWORD:
-            if key_string.startswith('@'):
+            if key_string.startswith("@"):
                 raise JMCValueError(
                     f"For '{key_string}' key, expected {verifier.value}, got {ArgType.SELECTOR.value}",
                     self.token,
                     tokenizer)
-            if ':' in key_string:
+            if ":" in key_string:
                 raise JMCValueError(
                     f"For '{key_string}' key, expected {verifier.value}, got {ArgType.SCOREBOARD.value}",
                     self.token,
@@ -144,40 +164,101 @@ class Arg:
         return self
 
 
-def find_arg_type(token: Token, tokenizer: Tokenizer) -> ArgType:
+def find_arg_type(tokens: list[Token], tokenizer: Tokenizer) -> ArgType:
     """
     Find type of the argument in form of token and return the ArgType
 
-    :param token: any token representing argument
+    :param tokens: List of tokens representing an argument
     :param tokenizer: Tokenizer
     :raises JMCValueError: Cannot find ArgType.FUNC type
     :return: Argument's type of the token
     """
-    if token.token_type == TokenType.FUNC:
-        return ArgType.ARROW_FUNC
-    if token.token_type == TokenType.PAREN_CURLY:
-        if re.match(r'^{\s*"', token.string) is not None:
-            return ArgType.JSON
-        return ArgType.JS_OBJECT
-    if token.token_type == TokenType.PAREN_SQUARE:
-        return ArgType.LIST
+    if not tokens:
+        raise ValueError("Trying to find type of empty argument")
+    if len(tokens) == 1:
+        if tokens[0].token_type == TokenType.PAREN_CURLY:
+            if re.match(r'^{\s*"', tokens[0].string) is not None:
+                return ArgType.JSON
+            return ArgType.JS_OBJECT
+        if tokens[0].token_type == TokenType.PAREN_SQUARE:
+            return ArgType.LIST
+        if tokens[0].token_type == TokenType.FUNC:
+            return ArgType.ARROW_FUNC
+        if tokens[0].token_type == TokenType.STRING:
+            return ArgType.STRING
 
-    if token.token_type == TokenType.KEYWORD:
-        if token.string.startswith(
-                DataPack.VARIABLE_SIGN) or ':' in token.string:
-            return ArgType.SCOREBOARD
-        if is_number(token.string):
+    if is_number(tokens[0].string):
+        if len(tokens) > 1:
+            raise JMCSyntaxException(
+                f"Unexpected {tokens[1].token_type.value} after integer in function argument",
+                tokens[1],
+                tokenizer)
+        return ArgType.INTEGER
+    if is_float(tokens[0].string):
+        if len(tokens) > 1:
+            raise JMCSyntaxException(
+                f"Unexpected {tokens[1].token_type.value} after float/decimal in function argument",
+                tokens[1],
+                tokenizer)
+        return ArgType.FLOAT
+
+    if tokens[0].string in {'-', '+'} and len(tokens) > 1:
+        if is_number(tokens[1].string):
+            if len(tokens) > 2:
+                raise JMCSyntaxException(
+                    f"Unexpected {tokens[2].token_type.value} after integer in function argument",
+                    tokens[2],
+                    tokenizer)
             return ArgType.INTEGER
-        if is_float(token.string):
+        if is_float(tokens[1].string):
+            if len(tokens) > 1:
+                raise JMCSyntaxException(
+                    f"Unexpected {tokens[2].token_type.value} after float/decimal in function argument",
+                    tokens[2],
+                    tokenizer)
             return ArgType.FLOAT
-        if token.string.startswith('@'):
+
+    if (
+        len(tokens) == 3
+        and
+        tokens[0].string.startswith(
+            DataPack.VARIABLE_SIGN)
+        and
+        tokens[1].string == ':'
+    ):
+        return ArgType.SCOREBOARD
+    if tokens[0].string.startswith("@"):
+        if len(tokens) == 1:
             return ArgType.SELECTOR
+        if tokens[1].token_type == TokenType.PAREN_SQUARE:
+            if len(tokens) > 2:
+                raise JMCSyntaxException(
+                    f"Unexpected token after target selector",
+                    tokens[2],
+                    tokenizer,
+                    suggestion="This can be a result from missing comma or missing quotation mark")
+            return ArgType.SELECTOR
+        raise JMCSyntaxException(
+            f"Expected '[' or nothing after target selector type (got {tokens[1].token_type})",
+            tokens[1],
+            tokenizer)
+
+    if tokens[0].token_type == TokenType.KEYWORD:
+        if len(tokens) == 1:
+            if tokens[0].string.startswith(DataPack.VARIABLE_SIGN):
+                return ArgType.SCOREBOARD
+            return ArgType.KEYWORD
+        for index, token in enumerate(tokens[1:]):
+            if token.token_type == tokens[index].token_type == TokenType.KEYWORD:
+                raise JMCSyntaxException(
+                    f"2 keywords are next to each other in function argument",
+                    token,
+                    tokenizer,
+                    suggestion="This can be a result from missing comma or missing quotation mark")
         return ArgType.KEYWORD
-    if token.token_type == TokenType.STRING:
-        return ArgType.STRING
 
     raise JMCValueError(
-        "Unknown argument type", token, tokenizer)
+        "Unknown argument type", tokens[-1], tokenizer)
 
 
 def verify_args(params: dict[str, ArgType], feature_name: str,
@@ -201,14 +282,26 @@ def verify_args(params: dict[str, ArgType], feature_name: str,
             f"{feature_name} takes {len(key_list)} positional arguments, got {len(args)}", token, tokenizer)
     for key, arg in zip(key_list, args):
         arg_type = find_arg_type(arg, tokenizer)
-        result[key] = Arg(arg, arg_type).verify(params[key], tokenizer, key)
+        arg_token = tokenizer.merge_tokens(arg)
+        result[key] = Arg(
+            arg_token,
+            arg_type).verify(
+            params[key],
+            tokenizer,
+            key)
     for key, kwarg in kwargs.items():
         if key not in key_list:
             raise JMCValueError(
-                f"{feature_name} got unexpected keyword argument '{key}'", kwarg, tokenizer,
+                f"{feature_name} got unexpected keyword argument '{key}'", kwarg[-1], tokenizer,
                 suggestion=f"""Available arguments are\n{', '.join(f"'{param}'" for param in params)}""")
         arg_type = find_arg_type(kwarg, tokenizer)
-        result[key] = Arg(kwarg, arg_type).verify(params[key], tokenizer, key)
+        kwarg_token = tokenizer.merge_tokens(kwarg)
+        result[key] = Arg(
+            kwarg_token,
+            arg_type).verify(
+            params[key],
+            tokenizer,
+            key)
     return result
 
 
@@ -219,7 +312,7 @@ def eval_expr(expr: str) -> str:
     :param expr: Expression string
     :return: String representation of result number
     """
-    return str(__eval(ast.parse(expr, mode='eval').body))
+    return str(__eval(ast.parse(expr, mode="eval").body))
 
 
 OPERATORS: dict[type, Callable[..., Any]] = {ast.Add: op.add, ast.Sub: op.sub, ast.Mult: op.mul,
@@ -261,6 +354,7 @@ class FormattedText:
     :param token: Token (only used for error)
     :param tokenizer: Tokenizer
     :param is_default_no_italic: Whether to set italic to False by default, defaults to False
+    :param is_allow_score_selector: Whether to allow score selector in the string, defaults to True
     :return: Raw JSON string
 
     .. example::
@@ -269,9 +363,10 @@ class FormattedText:
     >>> str(FormattedText("&<red,bold>Name", token, tokenizer, is_default_no_italic=True))
     '{"text":"Name", "color":"red", "bold":true, "italic":false}'
     """
-    __slots__ = ('raw_text', 'current_json', 'result',
-                 'bracket_content', 'token', 'tokenizer',
-                 'current_color', 'is_default_no_italic', 'datapack')
+    __slots__ = ("raw_text", "current_json", "result",
+                 "bracket_content", "token", "tokenizer",
+                 "current_color", "is_default_no_italic", "datapack",
+                 "is_allow_score_selector")
 
     SIGN = "&"
     OPEN_BRACKET = "<"
@@ -302,12 +397,13 @@ class FormattedText:
     }
 
     def __init__(self, raw_text: str, token: Token,
-                 tokenizer: Tokenizer, datapack: DataPack, *, is_default_no_italic: bool = False) -> None:
+                 tokenizer: Tokenizer, datapack: DataPack, *, is_default_no_italic: bool = False, is_allow_score_selector: bool = True) -> None:
         self.raw_text = raw_text
         self.token = token
         self.tokenizer = tokenizer
         self.datapack = datapack
         self.is_default_no_italic = is_default_no_italic
+        self.is_allow_score_selector = is_allow_score_selector
         self.bracket_content = ""
         self.result: list[SIMPLE_JSON_TYPE] = []
         self.current_json: SIMPLE_JSON_TYPE = {"text": ""}
@@ -345,93 +441,115 @@ class FormattedText:
                 value = False
                 prop = prop[1:]
             if prop in {
-                'dark_red',
-                'red',
-                'gold',
-                'yellow',
-                'dark_green',
-                'green',
-                'aqua',
-                'dark_aqua',
-                'blue',
-                'light_purple',
-                'dark_purple',
-                'white',
-                'gray',
-                'dark_gray',
-                'black',
-                'reset',
+                "dark_red",
+                "red",
+                "gold",
+                "yellow",
+                "dark_green",
+                "green",
+                "aqua",
+                "dark_aqua",
+                "blue",
+                "light_purple",
+                "dark_purple",
+                "white",
+                "gray",
+                "dark_gray",
+                "black",
+                "reset",
             }:
-                if 'color' in self.current_json:
+                if "color" in self.current_json:
                     raise JMCValueError(
-                        f'color({prop}) used twice in formatted text', self.token, self.tokenizer)
+                        f"color({prop}) used twice in formatted text", self.token, self.tokenizer)
 
-                self.current_json['color'] = prop
+                self.current_json["color"] = prop
                 self.current_color = prop
                 if not value:
                     raise JMCValueError(
-                        f'Color({prop}) cannot be false', self.token, self.tokenizer)
+                        f"Color({prop}) cannot be false", self.token, self.tokenizer)
                 continue
 
-            if prop in {'bold', 'italic', 'underlined',
-                        'strikethrough', 'obfuscated'}:
+            if prop in {"bold", "italic", "underlined",
+                        "strikethrough", "obfuscated"}:
                 self.current_json[prop] = value
                 continue
 
-            if prop.startswith('#') and len(prop) == 7:
-                if 'color' in self.current_json:
+            if prop.startswith("#") and len(prop) == 7:
+                if "color" in self.current_json:
                     raise JMCValueError(
-                        f'color({prop}) used twice in formatted text', self.token, self.tokenizer)
+                        f"color({prop}) used twice in formatted text", self.token, self.tokenizer)
 
-                self.current_json['color'] = prop
+                self.current_json["color"] = prop
                 if not value:
                     raise JMCValueError(
-                        f'Color({prop}) cannot be false', self.token, self.tokenizer)
+                        f"Color({prop}) cannot be false", self.token, self.tokenizer)
                 continue
 
-            if prop.startswith('$'):
-                if 'selector' in self.current_json:
+            if prop.startswith(DataPack.VARIABLE_SIGN):
+                if "selector" in self.current_json:
                     raise JMCValueError(
-                        f'selector used with score in formatted text', self.token, self.tokenizer)
-                if 'score' in self.current_json:
+                        f"selector used with score in formatted text", self.token, self.tokenizer)
+                if "score" in self.current_json:
                     raise JMCValueError(
-                        f'score used twice in formatted text', self.token, self.tokenizer)
-                self.current_json['score'] = {
+                        f"score used twice in formatted text", self.token, self.tokenizer)
+                self.current_json["score"] = {
                     "name": prop, "objective": self.datapack.var_name}
+                if not value:
+                    raise JMCValueError(
+                        f"Score cannot be false", self.token, self.tokenizer)
                 continue
 
-            if prop.count(':') == 1:
-                if 'selector' in self.current_json:
+            if prop.count(":") == 1:
+                if "selector" in self.current_json:
                     raise JMCValueError(
-                        f'selector used with score in formatted text', self.token, self.tokenizer)
-                if 'score' in self.current_json:
+                        f"selector used with score in formatted text", self.token, self.tokenizer)
+                if "score" in self.current_json:
                     raise JMCValueError(
-                        f'score used twice in formatted text', self.token, self.tokenizer)
-                objective, name = prop.split(':')
-                self.current_json['score'] = {
-                    "name": objective, "objective": name}
+                        f"score used twice in formatted text", self.token, self.tokenizer)
+                objective, name = prop.split(":")
+                self.current_json["score"] = {
+                    "name": name, "objective": objective}
+                if not value:
+                    raise JMCValueError(
+                        f"Score cannot be false", self.token, self.tokenizer)
                 continue
 
-            if prop.startswith('@'):
-                if 'score' in self.current_json:
+            if prop.startswith("@"):
+                if "score" in self.current_json:
                     raise JMCValueError(
-                        f'score used with selector in formatted text', self.token, self.tokenizer)
-                if 'selector' in self.current_json:
+                        f"score used with selector in formatted text", self.token, self.tokenizer)
+                if "selector" in self.current_json:
                     raise JMCValueError(
-                        f'selector used with selector in formatted text', self.token, self.tokenizer)
-                self.current_json['selector'] = prop
+                        f"selector used with selector in formatted text", self.token, self.tokenizer)
+                self.current_json["selector"] = prop
+                if not value:
+                    raise JMCValueError(
+                        f"Selector cannot be false", self.token, self.tokenizer)
                 continue
 
             raise JMCValueError(
                 f"Unknown property '{prop}'", self.token, self.tokenizer)
 
-        if 'color' not in self.current_json and self.current_color:
-            self.current_json['color'] = self.current_color
+        if "color" not in self.current_json and self.current_color:
+            self.current_json["color"] = self.current_color
 
-        if 'score' in self.current_json or 'selector' in self.current_json:
-            del self.current_json['text']
+        if "score" in self.current_json or "selector" in self.current_json:
+            if not self.is_allow_score_selector:
+                if "score" in self.current_json:
+                    raise JMCValueError(
+                        f"score is not allowed in this context in formatted text", self.token, self.tokenizer)
+                else:
+                    raise JMCValueError(
+                        f"selector is not allowed in this context in formatted text", self.token, self.tokenizer)
+            del self.current_json["text"]
+
+            tmp_json: SIMPLE_JSON_TYPE = {"text": ""}
+            for prop_, value_ in self.current_json.items():
+                if prop_ in {"bold", "italic", "underlined",
+                             "strikethrough", "obfuscated", "color"}:
+                    tmp_json[prop_] = value_
             self.result.append(self.current_json)
-            self.current_json = {"text": ""}
+            self.current_json = tmp_json
 
     def __parse_code(self, char: str) -> None:
         """
@@ -445,34 +563,34 @@ class FormattedText:
                 f"Unknown code format '{char}'", self.token, self.tokenizer)
 
         if prop in {
-            'dark_red',
-            'red',
-            'gold',
-            'yellow',
-            'dark_green',
-            'green',
-            'aqua',
-            'dark_aqua',
-            'blue',
-            'light_purple',
-            'dark_purple',
-            'white',
-            'gray',
-            'dark_gray',
-            'black',
-            'reset',
+            "dark_red",
+            "red",
+            "gold",
+            "yellow",
+            "dark_green",
+            "green",
+            "aqua",
+            "dark_aqua",
+            "blue",
+            "light_purple",
+            "dark_purple",
+            "white",
+            "gray",
+            "dark_gray",
+            "black",
+            "reset",
         }:
-            if 'color' in self.current_json:
+            if "color" in self.current_json:
                 raise JMCValueError(
-                    f'color({prop}) used twice in formatted text', self.token, self.tokenizer)
+                    f"color({prop}) used twice in formatted text", self.token, self.tokenizer)
 
-            self.current_json['color'] = prop
+            self.current_json["color"] = prop
             self.current_color = prop
 
-        if prop in {'bold', 'italic', 'underlined',
-                    'strikethrough', 'obfuscated'}:
+        if prop in {"bold", "italic", "underlined",
+                    "strikethrough", "obfuscated"}:
             if self.current_color:
-                self.current_json['color'] = self.current_color
+                self.current_json["color"] = self.current_color
             self.current_json[prop] = True
 
     def __parse(self) -> None:
@@ -493,23 +611,24 @@ class FormattedText:
 
             if is_expect_color_code:
                 is_expect_color_code = False
-                if char == self.OPEN_BRACKET:
-                    is_bracket_format = True
-                    continue
                 if char == self.SIGN:
                     if isinstance(self.current_json["text"], str):
-                        self.current_json["text"] += char
+                        self.current_json["text"] += char  # type: ignore
                         continue
+                if char == self.OPEN_BRACKET:
+                    self.__push()
+                    is_bracket_format = True
+                    continue
+                self.__push()
                 self.__parse_code(char)
                 continue
 
             if char == self.SIGN:
-                self.__push()
                 is_expect_color_code = True
                 continue
 
             if isinstance(self.current_json["text"], str):
-                self.current_json["text"] += char
+                self.current_json["text"] += char  # type: ignore
 
         if is_bracket_format:
             raise JMCValueError(
@@ -520,19 +639,55 @@ class FormattedText:
             raise JMCValueError(
                 f"Unexpected trailing '{self.SIGN}'", self.token, self.tokenizer, suggestion=f"Remove last '{self.SIGN}'")
 
+    @classmethod
+    def empty(cls, tokenizer: Tokenizer,
+              datapack: DataPack) -> "FormattedText":
+        return cls('', Token.empty(), tokenizer, datapack)
+
     def __bool__(self) -> bool:
         return bool(self.result) and ("text" in self.result[0])
 
     def __str__(self) -> str:
-        if len(self.result) == 1:
-            if self.is_default_no_italic and 'italic' not in self.result[0]:
-                self.result[0]['italic'] = False
-            return json.dumps(self.result[0])
-
         if not self.result:
-            return ''
+            return '""'
+
+        if len(self.result) == 1:
+            if self.is_default_no_italic and "italic" not in self.result[0]:
+                self.result[0]["italic"] = False
+            return json.dumps(self.result[0], separators=(",", ":"))
+
+        if self.is_default_no_italic and (
+                "italic" not in self.result[0] or not self.result[0]["italic"]):
+            self.result[0]["italic"] = False
+            return json.dumps(self.result, separators=(",", ":"))
+
         return json.dumps(
-            [{"italic": False} if self.is_default_no_italic else ""] + self.result)
+            [{"text": "", "italic": False} if self.is_default_no_italic else ""] + self.result, separators=(",", ":"))
 
     def __repr__(self) -> str:
         return f"FormattedText(raw_text={repr(self.raw_text)}, result={repr(json.dumps(self.result))})"
+
+
+ZERO_TO_Z_LENGTH = 36
+
+
+def __custom_hash(text: str):
+    hashed = 0
+    for ch in text:
+        hashed = (hashed * 281 ^ ord(ch) * 997) & 0xFFFFFFFF
+    return hashed
+
+
+def hash_string_to_string(string: str, length: int) -> str:
+    number: int = __custom_hash(string) % (ZERO_TO_Z_LENGTH**length)
+    if number == 0:
+        digits: list[int] = [0]
+    else:
+        digits = []
+        for _ in range(length):
+            digits.append(int(number % ZERO_TO_Z_LENGTH))
+            number //= ZERO_TO_Z_LENGTH
+        digits = digits[::-1]
+
+    return ''.join(str(digit) if 0 <= digit <= 9 else chr(
+        digit - 10 + 97) for digit in digits)
