@@ -9,6 +9,7 @@ if TYPE_CHECKING:
 
 logger = Logger(__name__)
 NEW_LINE = "\n"
+TAB = "\t"
 
 
 def log(self: object, args: tuple):
@@ -67,21 +68,23 @@ def error_msg(message: str, token: "Token|None", tokenizer: "Tokenizer", col_len
     try:
         msgs_ = tokenizer.file_string.split(NEW_LINE)
         max_space = len(str(display_line + 1))
+        line_ = overide_file_str(msgs_[display_line - 1])
         if entire_line:
-            line_ = overide_file_str(msgs_[display_line - 1])
+            tab_count = line_.count(TAB)
             msg = f"""In {tokenizer.file_path}
 {message} at line {line}.
-{display_line-1}{" "*(max_space-len(str(display_line - 1)))} |{msgs_[display_line-2] if display_line > 1 else ""}
-{display_line}{" "*(max_space-len(str(display_line)))} |{line_}
-{" "*(col+max_space+1)}{"^"*(len(line_)-col+1)}
-{display_line+1} |{msgs_[display_line] if display_line < len(msgs_) else ""}"""
+{display_line-1}{" "*(max_space-len(str(display_line - 1)))} |{msgs_[display_line-2].replace(TAB, "    ") if display_line > 1 else ""}
+{display_line}{" "*(max_space-len(str(display_line)))} |{line_.replace(TAB, "    ")}
+{" "*(col+max_space+3*tab_count+1)}{"^"*(len(line_)-col+1)}
+{display_line+1} |{msgs_[display_line].replace(TAB, "    ") if display_line < len(msgs_) else ""}"""
         else:
+            tab_count = line_[:col - 1].count(TAB)
             msg = f"""In {tokenizer.file_path}
 {message} at line {line} col {col}.
-{display_line-1}{" "*(max_space-len(str(display_line - 1)))} |{msgs_[display_line-2] if display_line > 1 else ""}
-{display_line}{" "*(max_space-len(str(display_line)))} |{overide_file_str(msgs_[display_line-1])}
-{" "*(col+max_space+1)}{"^"*(display_col-col)}
-{display_line+1} |{msgs_[display_line] if display_line < len(msgs_) else ""}"""
+{display_line-1}{" "*(max_space-len(str(display_line - 1)))} |{msgs_[display_line-2].replace(TAB, "    ") if display_line > 1 else ""}
+{display_line}{" "*(max_space-len(str(display_line)))} |{line_.replace(TAB, "    ")}
+{" "*(col+max_space+3*tab_count+1)}{"^"*(display_col-col)}
+{display_line+1} |{msgs_[display_line].replace(TAB, "    ") if display_line < len(msgs_) else ""}"""
     except IndexError as error:
         logger.critical(
             f"Error happens at wrong file: {tokenizer.file_path=}, {line=}, {col=}")
