@@ -114,11 +114,14 @@ class Lexer:
         :raises JMCSyntaxException: Path in `@import` is invalid
         :raises JMCSyntaxException: _description_
         """
+        if file_path in self.datapack._imported:
+            return
+        self.datapack._imported.add(file_path)
         logger.info(f"Parsing file: {file_path}")
         file_path_str = file_path.resolve().as_posix()
         if _test_file is None:
             try:
-                with file_path.open("r") as file:
+                with file_path.open("r", encoding="utf-8") as file:
                     raw_string = file.read()
             except FileNotFoundError as error:
                 raise JMCFileNotFoundError(
@@ -204,12 +207,6 @@ class Lexer:
                 self.parse_file(file_path=new_path)
                 self.__update_load(file_path_str, raw_string)
             else:
-                # if not is_load:
-                #     raise JMCSyntaxException(
-                # f"Command({command[0].string}) found inside non-load file
-                # (Not main JMC file)", command[1], tokenizer,
-                # suggestion="Consider putting it in a new function")
-
                 self.datapack.load_function.append(command)
 
         self.parse_current_load()
@@ -237,7 +234,7 @@ class Lexer:
                 "Expected keyword(function's name)", command[1], tokenizer)
         if command[2].string != "()":
             raise JMCSyntaxException(
-                "Expected empty round parenthesis", command[2], tokenizer)
+                "Expected empty round bracket, `()`", command[2], tokenizer)
         if command[3].token_type != TokenType.PAREN_CURLY:
             raise JMCSyntaxException(
                 "Expected {", command[3], tokenizer, display_col_length=False)
@@ -295,7 +292,7 @@ class Lexer:
                 "Expected keyword(JSON file's type)", command[1], tokenizer)
         if len(command) < 3:
             raise JMCSyntaxException(
-                "Expected round parenthesis(JSON file's path)", command[1], tokenizer, col_length=True)
+                "Expected round bracket(JSON file's path)", command[1], tokenizer, col_length=True)
         if command[2].string == "()":
             raise JMCSyntaxException(
                 "Expected JSON file's path in the bracket", command[1], tokenizer)
@@ -316,7 +313,7 @@ class Lexer:
         if json_type not in JSON_FILE_TYPES and not json_type.startswith(
                 "tags/"):
             raise MinecraftSyntaxWarning(
-                f"Unrecognized JSON file's type({json_type})", command[2], tokenizer
+                f"Unrecognized JSON file's type({json_type})", command[1], tokenizer
             )
 
         json_name = prefix + convention_jmc_to_mc(
