@@ -55,9 +55,10 @@ FIRST_ARGUMENTS_EXCEPTION = {
     "bossbar": {"execute"},
     "title": {"title"},
     "item": {"summon"},
-    "loot": {"loot"},
+    "loot": {"loot", "give"},
     "data": {"execute"},
-    "playsound": {"weather"}
+    "playsound": {"weather"},
+    "particle": {"item"}
 }
 """Dictionary of (FIRST_ARGUMENTS that can also be used as normal argument in a command) and (those commands)
 
