@@ -56,8 +56,8 @@ class Token:
     line: int
     col: int
     string: str
-    _macro_length: int = 0
     """The string representation (including parentheses, excluding quotation mark)"""
+    _macro_length: int = 0
 
     # def __new__(cls: type["Token"], token_type: TokenType, line: int, col: int, string: str) -> "Token":
     #     return super().__new__(cls)
@@ -400,7 +400,7 @@ class Tokenizer:
         self.is_comment = False
         if self.state == TokenType.STRING:
             raise JMCSyntaxException(
-                "String literal contains an unescaped line break.", None, self, entire_line=True, display_col_length=False, suggestion="Consider changing '\\n' to '\\\\n'")
+                "String literal contains an unescaped line break.", None, self, entire_line=True, display_col_length=False)  # suggestion="Consider changing '\\n' to '\\\\n'")
         if self.state == TokenType.COMMENT:
             self.state = None
         elif self.state == TokenType.KEYWORD or self.state == TokenType.OPERATOR:
@@ -562,7 +562,7 @@ class Tokenizer:
                 self.append_keywords()
             else:
                 raise JMCSyntaxException(
-                    "Expected semicolon(;)", self.keywords[-1], self, col_length=True)
+                    "Expected semicolon(;)", self.keywords[-1], self)
 
         if not expect_semicolon:
             if self.token_str != "":
@@ -600,12 +600,13 @@ class Tokenizer:
         return tokens
 
     def find_token(self, tokens: list[Token],
-                   string: str) -> list[list[Token]]:
+                   string: str, allow_string_token: bool = False) -> list[list[Token]]:
         """
         Split list of tokens by token that match the string
 
         :param tokens: List of token
         :param string: String to match for splitting
+        :param allow_string_token: Whether to allow string token
         :return: List of list of tokens
 
         .. example::
@@ -615,7 +616,13 @@ class Tokenizer:
         result: list[list[Token]] = []
         token_array: list[Token] = []
         for token in tokens:
-            if token.string == string:
+            if allow_string_token:
+                is_append_result = token.string == string
+
+            else:
+                is_append_result = token.string == string and token.token_type != TokenType.STRING
+
+            if is_append_result:
                 result.append(token_array)
                 token_array = []
             else:
