@@ -1,4 +1,6 @@
+"""Utility for compiling"""
 import functools
+from random import Random
 import re
 from json import JSONEncoder, dumps
 from typing import TYPE_CHECKING, Any
@@ -9,11 +11,11 @@ if TYPE_CHECKING:
     from .tokenizer import Token, Tokenizer
 
 
-class __SingleTonMeta(type):
+class SingleTonMeta(type):
     """
     Metaclass for singleton
     """
-    _instances: dict["__SingleTonMeta", Any] = {}
+    _instances: dict["SingleTonMeta", Any] = {}
 
     def __call__(cls, *args, **kwargs):
         if cls not in cls._instances:
@@ -21,7 +23,7 @@ class __SingleTonMeta(type):
         return cls._instances[cls]
 
 
-class SingleTon(metaclass=__SingleTonMeta):
+class SingleTon(metaclass=SingleTonMeta):
     """
     Super class for a singleton class
 
@@ -60,10 +62,21 @@ def is_float(string: str) -> bool:
 
 def is_connected(current_token: "Token", previous_token: "Token") -> bool:
     """Whether 2 tokens are next to each other"""
+    if not previous_token._macro_length:
+        return (
+            previous_token.line == current_token.line and
+            previous_token.col +
+            previous_token.length == current_token.col
+        )
     return (
         previous_token.line == current_token.line and
-        previous_token.col +
-        previous_token.length == current_token.col
+        (
+            previous_token.col +
+            previous_token.length == current_token.col
+            or
+            previous_token.col +
+            previous_token._macro_length == current_token.col
+        )
     )
 
 
@@ -77,41 +90,41 @@ def __parse_to_string(
     :return: Dictionary of key(key) and value(string or boolean)
     """
     json: dict[str, str | bool | dict[str, str]] = {}
-    if token.string == '()':
+    if token.string == "()":
         return json
 
-    for kwargs in token.string[1:-1].split(','):
-        if '=' not in kwargs:
+    for kwargs in token.string[1:-1].split(","):
+        if "=" not in kwargs:
             raise JMCSyntaxException(
                 "'=' not found in .toString argument", token, tokenizer)
-        key, value, *extras = kwargs.split('=')
+        key, value, *extras = kwargs.split("=")
         key = key.strip()
         if extras:
             raise JMCSyntaxException(
                 f"Too many '=' found in .toString argument (got {len(extras)+1})", token, tokenizer)
         if value in {'""', "''"}:
             raise JMCSyntaxException(
-                ".toString function accepts keyword as arguments, not string", token, tokenizer, suggestion=f"Do not use empty string as argument")
+                ".toString function accepts keyword as arguments, not string", token, tokenizer, suggestion="Do not use empty string as argument")
         if value[0] in {'"', "'"}:
             raise JMCSyntaxException(
                 ".toString function accepts keyword as arguments, not string", token, tokenizer, suggestion=f"Use '{value[1:-1]}' instead of '{value}'")
-        if value[0] in {'{', '(', '['}:
+        if value[0] in {"{", "(", "["}:
             raise JMCSyntaxException(
-                "Brackets are not supported in .toString", token, tokenizer, suggestion="clickEvent and hoverEvent are not supported, use vanila JSON instead.")
+                "Brackets are not supported in .toString", token, tokenizer, suggestion="clickEvent and hoverEvent are not supported, use normal minecraft JSON instead.")
 
-        if key in {'font', 'color'}:
+        if key in {"font", "color"}:
             json[key] = value
-        elif key in {'bold', 'italic', 'underlined', 'strikethrough', 'obfuscated'}:
-            if value not in {'true', 'false'}:
+        elif key in {"bold", "italic", "underlined", "strikethrough", "obfuscated"}:
+            if value not in {"true", "false"}:
                 raise JMCSyntaxException(
                     f"value of {key} .toString must be either 'true' or 'false'", token, tokenizer)
-            json[key] = value == 'true'
-        elif key in {'clickEvent', 'hoverEvent'}:
+            json[key] = value == "true"
+        elif key in {"clickEvent", "hoverEvent"}:
             raise JMCSyntaxException(
-                f"clickEvent and hoverEvent are not supported in .toString", token, tokenizer, suggestion="Use vanila JSON instead")
-        elif key == 'text':
+                "clickEvent and hoverEvent are not supported in .toString", token, tokenizer, suggestion="Use normal minecraft JSON instead")
+        elif key == "text":
             raise JMCSyntaxException(
-                f"'text' key is incompatible with 'score' in .toString", token, tokenizer)
+                "'text' key is incompatible with 'score' in .toString", token, tokenizer)
         else:
             raise JMCSyntaxException(
                 f"Unrecognized key in .toString (got {key})", token, tokenizer, suggestion="Avaliable keys are 'font', 'color', 'bold', 'italic', 'underlined', 'strikethrough', 'obfuscated'")
@@ -119,24 +132,24 @@ def __parse_to_string(
 
 
 def __search_to_string(match: re.Match, token: "Token",
-                       VAR_NAME: str, tokenizer: "Tokenizer") -> str:
+                       var_name: str, tokenizer: "Tokenizer") -> str:
     """
     Function for regex.subn
 
     :param match: Match object
     :param token: Token to search for `toString`
-    :param VAR_NAME: `DataPack.VARNAME`
+    :param var_name: `DataPack.VARNAME`
     :param tokenizer: token's tokenizer
     :return: JSON formatted string
     """
     var: str = match.group(1)
     properties = __parse_to_string(token, tokenizer)
-    properties["score"] = {"name": var, "objective": VAR_NAME}
-    return dumps(properties)
+    properties["score"] = {"name": var, "objective": var_name}
+    return dumps(properties, separators=(",", ":"))
 
 
 def search_to_string(last_str: str, token: "Token",
-                     VAR_NAME: str, tokenizer: "Tokenizer") -> tuple[str, bool]:
+                     var_name: str, tokenizer: "Tokenizer") -> tuple[str, bool]:
     """
     Find `toString` in a last_str
 
@@ -147,7 +160,7 @@ def search_to_string(last_str: str, token: "Token",
     :return: Tuple of New string and Whether `toString` is found
     """
     new_str, count = re.subn(
-        r'(\$[A-z0-9\-\.\_]+)\.toString$', lambda match: __search_to_string(match, token, VAR_NAME, tokenizer), last_str)
+        r"(\$[A-z0-9\-\.\_]+)\.toString$", lambda match: __search_to_string(match, token, var_name, tokenizer), last_str)
     if count:
         return new_str, True
     return last_str, False
@@ -213,4 +226,12 @@ def convention_jmc_to_mc(token: "Token", tokenizer: "Tokenizer",
         raise MinecraftSyntaxWarning(
             f"Invalid character detected in '{string}'", token, tokenizer
         )
-    return string.replace('.', '/')
+    return string.replace(".", "/")
+
+
+def get_mc_uuid(seed: Any) -> str:
+    random_instance = Random(seed)
+
+    def rand_java_int() -> int:
+        return random_instance.randint(-2147483648, 2147483647)
+    return f"[I;{rand_java_int()},{rand_java_int()},{rand_java_int()},{rand_java_int()}]"
